@@ -72,24 +72,23 @@ level API.
 
 = License
 
-Copyright (C) 2003,2006 Daiki Ueno
+Copyright (C) 2003,2006,2007,2008,2009 Daiki Ueno
 
 This file is a part of Ruby-GPGME.
 
-This program is free software; you can redistribute it and/or modify 
-it under the terms of the GNU General Public License as published by 
-the Free Software Foundation; either version 2, or (at your option)  
-any later version.                                                   
+Ruby-GPGME is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-This program is distributed in the hope that it will be useful,      
-but WITHOUT ANY WARRANTY; without even the implied warranty of       
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the        
-GNU General Public License for more details.                         
+Ruby-GPGME is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License    
-along with GNU Emacs; see the file COPYING.  If not, write to the    
-Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-Boston, MA 02110-1301, USA.
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 =end
 module GPGME
 end
@@ -124,29 +123,30 @@ def GPGME.decrypt(cipher, *args_options)
   args, options = split_args(args_options)
   plain = args[0]
 
-  ctx = GPGME::Ctx.new(options)
-  cipher_data = input_data(cipher)
-  plain_data = output_data(plain)
-  begin
-    ctx.decrypt_verify(cipher_data, plain_data)
-  rescue GPGME::Error::UnsupportedAlgorithm => exc
-    exc.algorithm = ctx.decrypt_result.unsupported_algorithm
-    raise exc
-  rescue GPGME::Error::WrongKeyUsage => exc
-    exc.key_usage = ctx.decrypt_result.wrong_key_usage
-    raise exc
-  end
-
-  verify_result = ctx.verify_result
-  if verify_result && block_given?
-    verify_result.signatures.each do |signature|
-      yield signature
+  GPGME::Ctx.new(options) do |ctx|
+    cipher_data = input_data(cipher)
+    plain_data = output_data(plain)
+    begin
+      ctx.decrypt_verify(cipher_data, plain_data)
+    rescue GPGME::Error::UnsupportedAlgorithm => exc
+      exc.algorithm = ctx.decrypt_result.unsupported_algorithm
+      raise exc
+    rescue GPGME::Error::WrongKeyUsage => exc
+      exc.key_usage = ctx.decrypt_result.wrong_key_usage
+      raise exc
     end
-  end
 
-  unless plain
-    plain_data.seek(0, IO::SEEK_SET)
-    plain_data.read
+    verify_result = ctx.verify_result
+    if verify_result && block_given?
+      verify_result.signatures.each do |signature|
+        yield signature
+      end
+    end
+
+    unless plain
+      plain_data.seek(0, IO::SEEK_SET)
+      plain_data.read
+    end
   end
 end
 
@@ -182,22 +182,23 @@ def GPGME.verify(sig, *args_options) # :yields: signature
   args, options = split_args(args_options)
   signed_text, plain = args
 
-  ctx = GPGME::Ctx.new(options)
-  sig_data = input_data(sig)
-  if signed_text
-    signed_text_data = input_data(signed_text)
-    plain_data = nil
-  else
-    signed_text_data = nil
-    plain_data = output_data(plain)
-  end
-  ctx.verify(sig_data, signed_text_data, plain_data)
-  ctx.verify_result.signatures.each do |signature|
-    yield signature
-  end
-  if !signed_text && !plain
-    plain_data.seek(0, IO::SEEK_SET)
-    plain_data.read
+  GPGME::Ctx.new(options) do |ctx|
+    sig_data = input_data(sig)
+    if signed_text
+      signed_text_data = input_data(signed_text)
+      plain_data = nil
+    else
+      signed_text_data = nil
+      plain_data = output_data(plain)
+    end
+    ctx.verify(sig_data, signed_text_data, plain_data)
+    ctx.verify_result.signatures.each do |signature|
+      yield signature
+    end
+    if !signed_text && !plain
+      plain_data.seek(0, IO::SEEK_SET)
+      plain_data.read
+    end
   end
 end
 
@@ -235,21 +236,22 @@ def GPGME.sign(plain, *args_options)
   args, options = split_args(args_options)
   sig = args[0]
 
-  ctx = GPGME::Ctx.new(options)
-  ctx.add_signer(*resolve_keys(options[:signers], true)) if options[:signers]
-  mode = options[:mode] || GPGME::SIG_MODE_NORMAL
-  plain_data = input_data(plain)
-  sig_data = output_data(sig)
-  begin
-    ctx.sign(plain_data, sig_data, mode)
-  rescue GPGME::Error::UnusableSecretKey => exc
-    exc.keys = ctx.sign_result.invalid_signers
-    raise exc
-  end
+  GPGME::Ctx.new(options) do |ctx|
+    ctx.add_signer(*resolve_keys(options[:signers], true, [:sign])) if options[:signers]
+    mode = options[:mode] || GPGME::SIG_MODE_NORMAL
+    plain_data = input_data(plain)
+    sig_data = output_data(sig)
+    begin
+      ctx.sign(plain_data, sig_data, mode)
+    rescue GPGME::Error::UnusableSecretKey => exc
+      exc.keys = ctx.sign_result.invalid_signers
+      raise exc
+    end
 
-  unless sig
-    sig_data.seek(0, IO::SEEK_SET)
-    sig_data.read
+    unless sig
+      sig_data.seek(0, IO::SEEK_SET)
+      sig_data.read
+    end
   end
 end
 
@@ -344,39 +346,46 @@ end
 # <i>options</i> are same as <code>GPGME::Ctx.new()</code> except for
 #
 # - <tt>:sign</tt> If <tt>true</tt>, it performs a combined sign and
-# encrypt operation.
+#   encrypt operation.
 # - <tt>:signers</tt> Signing keys.  If specified, it is an array
 #   whose elements are a GPGME::Key object or a string.
+# - <tt>:always_trust</tt> Setting this to <tt>true</tt> specifies all
+#   the recipients should be trusted.
 #
 def GPGME.encrypt(recipients, plain, *args_options)
   raise ArgumentError, 'wrong number of arguments' if args_options.length > 3
   args, options = split_args(args_options)
   cipher = args[0]
-  recipient_keys = recipients ? resolve_keys(recipients, false) : nil
+  recipient_keys = recipients ? resolve_keys(recipients, false, [:encrypt]) : nil
 
-  ctx = GPGME::Ctx.new(options)
-  plain_data = input_data(plain)
-  cipher_data = output_data(cipher)
-  begin
-    if options[:sign]
-      if options[:signers]
-        ctx.add_signer(*resolve_keys(options[:signers], true))
+  GPGME::Ctx.new(options) do |ctx|
+    plain_data = input_data(plain)
+    cipher_data = output_data(cipher)
+    begin
+      flags = 0
+      if options[:always_trust]
+        flags |= GPGME::ENCRYPT_ALWAYS_TRUST
       end
-      ctx.encrypt_sign(recipient_keys, plain_data, cipher_data)
-    else
-      ctx.encrypt(recipient_keys, plain_data, cipher_data)
+      if options[:sign]
+        if options[:signers]
+          ctx.add_signer(*resolve_keys(options[:signers], true, [:sign]))
+        end
+        ctx.encrypt_sign(recipient_keys, plain_data, cipher_data, flags)
+      else
+        ctx.encrypt(recipient_keys, plain_data, cipher_data, flags)
+      end
+    rescue GPGME::Error::UnusablePublicKey => exc
+      exc.keys = ctx.encrypt_result.invalid_recipients
+      raise exc
+    rescue GPGME::Error::UnusableSecretKey => exc
+      exc.keys = ctx.sign_result.invalid_signers
+      raise exc
     end
-  rescue GPGME::Error::UnusablePublicKey => exc
-    exc.keys = ctx.encrypt_result.invalid_recipients
-    raise exc
-  rescue GPGME::Error::UnusableSecretKey => exc
-    exc.keys = ctx.sign_result.invalid_signers
-    raise exc
-  end
 
-  unless cipher
-    cipher_data.seek(0, IO::SEEK_SET)
-    cipher_data.read
+    unless cipher
+      cipher_data.seek(0, IO::SEEK_SET)
+      cipher_data.read
+    end
   end
 end
 
@@ -403,13 +412,14 @@ def GPGME.list_keys(*args_options) # :yields: key
   raise ArgumentError, 'wrong number of arguments' if args_options.length > 3
   args, options = split_args(args_options)
   pattern, secret_only = args
-  ctx = GPGME::Ctx.new
-  if block_given?  
-    ctx.each_key(pattern, secret_only || false) do |key|
-      yield key
+  GPGME::Ctx.new do |ctx|
+    if block_given?  
+      ctx.each_key(pattern, secret_only || false) do |key|
+        yield key
+      end
+    else
+      ctx.keys(pattern, secret_only || false)
     end
-  else
-    ctx.keys(pattern, secret_only || false)
   end
 end
 
@@ -440,12 +450,13 @@ def GPGME.export(*args_options)
   args, options = split_args(args_options)
   pattern, key = args[0]
   key_data = output_data(key)
-  ctx = GPGME::Ctx.new(options)
-  ctx.export_keys(pattern, key_data)
+  GPGME::Ctx.new(options) do |ctx|
+    ctx.export_keys(pattern, key_data)
 
-  unless key
-    key_data.seek(0, IO::SEEK_SET)
-    key_data.read
+    unless key
+      key_data.seek(0, IO::SEEK_SET)
+      key_data.read
+    end
   end
 end
 
@@ -473,9 +484,10 @@ def GPGME.import(*args_options)
   args, options = split_args(args_options)
   key = args[0]
   key_data = input_data(key)
-  ctx = GPGME::Ctx.new(options)
-  ctx.import_keys(key_data)
-  ctx.import_result
+  GPGME::Ctx.new(options) do |ctx|
+    ctx.import_keys(key_data)
+    ctx.import_result
+  end
 end
 
 module GPGME
@@ -494,14 +506,18 @@ module GPGME
   end
   module_function :split_args
 
-  def resolve_keys(keys_or_names, secret_only)
-    ctx = GPGME::Ctx.new
+  def resolve_keys(keys_or_names, secret_only, purposes = Array.new)
     keys = Array.new
     keys_or_names.each do |key_or_name|
       if key_or_name.kind_of? Key
         keys << key_or_name
       elsif key_or_name.kind_of? String
-        keys += ctx.keys(key_or_name, secret_only)
+        GPGME::Ctx.new do |ctx|
+          key = ctx.keys(key_or_name, secret_only).find {|k|
+            k.usable_for?(purposes)
+          }
+          keys << key if key
+        end
       end
     end
     keys
@@ -709,12 +725,16 @@ module GPGME
     alias hash_algo_name gpgme_hash_algo_name
   end
 
-  def engine_check_version
-    err = GPGME::gpgme_engine_check_version
+  # Verify that the engine implementing the protocol <i>proto</i> is
+  # installed in the system.
+  def engine_check_version(proto)
+    err = GPGME::gpgme_engine_check_version(proto)
     exc = GPGME::error_to_exception(err)
     raise exc if exc
   end
+  module_function :engine_check_version
 
+  # Return a list of info structures of enabled engines.
   def engine_info
     rinfo = Array.new
     GPGME::gpgme_get_engine_info(rinfo)
@@ -722,6 +742,19 @@ module GPGME
   end
   module_function :engine_info
 
+  # Change the default configuration of the crypto engine implementing
+  # protocol <i>proto</i>.
+  #
+  # <i>file_name</i> is the file name of the executable program
+  # implementing the protocol.
+  # <i>home_dir</i> is the directory name of the configuration directory.
+  def set_engine_info(proto, file_name, home_dir)
+    err = GPGME::gpgme_set_engine_info(proto, file_name, home_dir)
+    exc = GPGME::error_to_exception(err)
+    raise exc if exc
+  end
+  module_function :set_engine_info
+
   # A class for managing data buffers.
   class Data
     BLOCK_SIZE = 4096
@@ -829,7 +862,7 @@ module GPGME
   class EngineInfo
     private_class_method :new
     
-    attr_reader :protocol, :file_name, :version, :req_version
+    attr_reader :protocol, :file_name, :version, :req_version, :home_dir
     alias required_version req_version
   end
 
@@ -882,7 +915,15 @@ module GPGME
                                       options[:progress_callback_value])
         end
       end
-      ctx
+      if block_given?
+        begin
+          yield ctx
+        ensure
+          GPGME::gpgme_release(ctx)
+        end
+      else
+        ctx
+      end
     end
 
     # Set the <i>protocol</i> used within this context.
@@ -952,7 +993,7 @@ keylist_mode=#{KEYLIST_MODE_NAMES[keylist_mode]}>"
     #      (0 ... $_.length).each do |i| $_[i] = ?0 end if $_
     #      system('stty echo')
     #    end
-    #    puts
+    #    $stderr.puts
     #  end
     #
     #  ctx.set_passphrase_callback(method(:passfunc))
@@ -1095,6 +1136,23 @@ keylist_mode=#{KEYLIST_MODE_NAMES[keylist_mode]}>"
     end
     alias delete delete_key
 
+    # Edit attributes of the key in the local key ring.
+    def edit_key(key, editfunc, hook_value = nil, out = Data.new)
+      err = GPGME::gpgme_op_edit(self, key, editfunc, hook_value, out)
+      exc = GPGME::error_to_exception(err)
+      raise exc if exc
+    end
+    alias edit edit_key
+
+    # Edit attributes of the key on the card.
+    def edit_card_key(key, editfunc, hook_value = nil, out = Data.new)
+      err = GPGME::gpgme_op_card_edit(self, key, editfunc, hook_value, out)
+      exc = GPGME::error_to_exception(err)
+      raise exc if exc
+    end
+    alias edit_card edit_card_key
+    alias card_edit edit_card_key
+
     # Decrypt the ciphertext and return the plaintext.
     def decrypt(cipher, plain = Data.new)
       err = GPGME::gpgme_op_decrypt(self, cipher, plain)
@@ -1159,7 +1217,7 @@ keylist_mode=#{KEYLIST_MODE_NAMES[keylist_mode]}>"
     end
 
     def sign_result
-      GPGME::gpgme_sign_result(self)
+      GPGME::gpgme_op_sign_result(self)
     end
 
     # Encrypt the plaintext in the data object for the recipients and
@@ -1207,6 +1265,14 @@ keylist_mode=#{KEYLIST_MODE_NAMES[keylist_mode]}>"
       caps
     end
 
+    def usable_for?(purposes)
+      unless purposes.kind_of? Array
+        purposes = [purposes]
+      end
+      return false if [:revoked, :expired, :disabled, :invalid].include? trust
+      return (purposes - capability).empty?
+    end
+
     def secret?
       @secret == 1
     end
@@ -1267,6 +1333,14 @@ capability=%s, subkeys=%s, uids=%s>",
       caps
     end
 
+    def usable_for?(purposes)
+      unless purposes.kind_of? Array
+        purposes = [purposes]
+      end
+      return false if [:revoked, :expired, :disabled, :invalid].include? trust
+      return (purposes - capability).empty?
+    end
+
     def secret?
       @secret == 1
     end
@@ -1373,7 +1447,9 @@ validity=#{VALIDITY_NAMES[validity]}, signatures=#{signatures.inspect}>"
   class Signature
     private_class_method :new
 
-    attr_reader :summary, :fpr, :status, :notations
+    attr_reader :summary, :fpr, :status, :notations, :wrong_key_usage
+    attr_reader :validity, :validity_reason
+    attr_reader :pka_trust, :pka_address
     alias fingerprint fpr
 
     def timestamp
